#define VERSION "V1.02"

#include <stdio.h>
#include <stdlib.h>     /* exit()*/
#include <string.h>
#include <ctype.h>      /* isalnum()*/


int Verbose;



void Parse(FILE *VAGfd, FILE *WBHfd){

 char VAGLine[1024];
 char *VAGPtr;
 char Group[10];
 char Column;
 char Text[100];
 int i;

 int GroupFlag = 0;
 int OldGroup = 0;
 int OldColumn = '5';

 int RawFlag = 0;
 int CodingFlag = 0;


 /* no informations available in VAG-Com labelfiles
  so write only the headline
 */
 fprintf(WBHfd, "\n[Info]\n\n");

 while(fgets(VAGLine, sizeof(VAGLine), VAGfd)){
	if(VAGLine[0] != ';') {

	 if(isdigit(VAGLine[0])){
		VAGPtr = VAGLine;
		
		i = 0;
		do {
		 Group[i++] = *VAGPtr++;
		}	while(isdigit(*VAGPtr));
		Group[i] = '\0';
		
		if(atoi(Group) != 0){
		
		 if(!GroupFlag){
			GroupFlag = 1;
			fprintf(WBHfd, "\n[Groups]\n");
			if(Verbose > 1) printf(" Writing groups.\n");
		 }
		
		 if(atoi(Group) != OldGroup){
			/* to add missing columns */
  		for(i = OldColumn + 1; i < '5'; i++)
			 fprintf(WBHfd, "G%i.%c=\n", OldGroup, i);
			OldGroup = atoi(Group);
			fprintf(WBHfd, "\n");
			OldColumn = '0';
		 }
		
		 while(!isdigit(*VAGPtr)) VAGPtr++;
		
		 if(*VAGPtr > '0'){
		  Column = *VAGPtr;
		  
		  /* to fill in not used columns*/
		  for(i = OldColumn + 1; i < Column; i++){
       fprintf(WBHfd, "G%u.%c=\n", atoi(Group), i);
			}
			OldColumn = Column;
		  
		  while(*VAGPtr++ != ',');
		  
		  /* first text part*/
		  i = 0;
		  do {
			 Text[i++] = *VAGPtr++;
			} while(((unsigned char)*VAGPtr > 13) && (*VAGPtr != ','));
		  
		  if((unsigned char)*VAGPtr > 13 ){
		  
		   VAGPtr++;

			 if(((unsigned char)*VAGPtr > 13) && (*VAGPtr != ',')){
				if(Text[i - 1] == '-') i--;
			  else Text[i++] = ' ';
		    do {
			   Text[i++] = *VAGPtr++;
			  } while(((unsigned char)*VAGPtr > 13) && (*VAGPtr != ','));
			 }
			}
		  Text[i] = '\0';
		  
			fprintf(WBHfd, "G%u.%c=%s\n", atoi(Group), Column, Text);
		 }
		} else {
		
		 if(!RawFlag){
		  RawFlag = 1;
		  fprintf(WBHfd, "\n[Raw]\n\n");
		  if(Verbose > 1) printf(" Writing raw.\n");
		 }

     while(!isdigit(*VAGPtr)) VAGPtr++;

		 i = 0;
     do {
			Text[i++] = *VAGPtr++;
		 } while(isdigit(*VAGPtr));
		 Text[i] = '\0';
		 Column = atoi(Text);

		 if(Column != 0){
			VAGPtr++;
			i = 0;
			do {
			 Text[i++] = *VAGPtr++;
      } while((unsigned char)*VAGPtr > 13 && (*VAGPtr != ','));
			Text[i] = '\0';
		  fprintf(WBHfd, "raw%u=%s\n", Column, Text);
		 }
		}
		
	 } else {

		if(VAGLine[0] == 'C'){
		 if(!CodingFlag){
      CodingFlag = 1;
		  fprintf(WBHfd, "\n[Coding]\n\n");
		  if(Verbose > 1) printf(" Writing coding.\n");
		 }


		 i = 0;
		 VAGPtr = &VAGLine[1];
		 do{
			Text[i++] = *VAGPtr++;
		 } while(isdigit(*VAGPtr));
		 Text[i] = '\0';

		 Column = atoi(Text);

		 if(Column > 0){
			while(*VAGPtr++ != ',');
		  i = 0;
		  do{
		 	 Text[i++] = *VAGPtr++;
		  } while((unsigned char)*VAGPtr > 13);
		  Text[i] = '\0';
		  fprintf(WBHfd, "C%u=%s\n", Column, Text);
		 }
		}

	 }
	}
 }

 /* no informations available in VAG-Com labelfiles*/
 /* so write only the headline*/
 fprintf(WBHfd, "\n;\n");
 fprintf(WBHfd, "; no informations about adaptions available\n");
 fprintf(WBHfd, ";\n\n");
 fprintf(WBHfd, "[Adaption]\n\n");
}




void Convert(char *VAG, char *WBH){

 FILE *VAGfd, *WBHfd;
 
 if(Verbose > 1) printf(" Open VAG-Com labelfile %s for reading.\n", VAG);
 VAGfd = fopen(VAG, "r");
 if(VAGfd){
  if(Verbose > 1) printf(" Open WBH-Diag labelfile %s for writing.\n", WBH);
	WBHfd = fopen(WBH, "w");
	if(WBHfd){

	 fprintf(WBHfd, ";\n");
	 fprintf(WBHfd, "; WBH-Diag labelfile\n");
	 fprintf(WBHfd, ";\n");
	 fprintf(WBHfd, "; created with VAGC2WBH %s\n", VERSION);
   fprintf(WBHfd, ";\n");
	 
	 Parse(VAGfd, WBHfd);

	 fclose(WBHfd);
	 if(Verbose > 1) printf(" WBH-Diag labelfile %s closed.\n", WBH);
	} else {
	 printf(" Can not open WBH-Diag labelfile %s !!!\n", WBH);
	}
	fclose(VAGfd);
	if(Verbose > 1) printf(" VAG-Com labelfile %s closed.\n", VAG);
 } else {
	printf(" Can not open VAG-Com labelfile %s !!!\n", VAG);
 }
}




void CreateWBHFilename(char *VAG, char *WBH){

 do {
	if(isalnum(*VAG)) *WBH++ = *VAG;
	VAG++;
 } while((*VAG != '\0') && (*VAG != '.'));
 *WBH = '\0';
 strcat(WBH, ".ini");
}




void Usage(){

 puts("");
 printf(" VAG-Com to WBH-Diag labelfile converter %s\n", VERSION);
 puts("");
 puts(" usage: VAGC2WBHD VAGLabelfile [WBHLabelfile] [-v]");
 puts("");
 puts(" VAGLabelfile : the filename of the VAG-Com labelfile");
 puts(" WBHLabelfile : optional, the filename of the WBH-Diag labelfile");
 puts(" -v           : optional, increase the verbosity");
 puts("");
 puts(" example: VAGC2WBHD 074-906-018-AXL");
 puts("");
 exit(1);
}




int main(int argc, char *argv[]){

 int i;
 char VAGFilename[1024];
 char WBHFilename[1024];


 if(argc < 2) Usage();

 Verbose = 0;
 VAGFilename[0] = '\0';
 VAGFilename[sizeof(VAGFilename) - 1] = '\0';
 WBHFilename[0] = '\0';
 WBHFilename[sizeof(WBHFilename) - 1] = '\0';

 for(i = 1; i < argc; i++){
	if(strstr(argv[i], "-v")) Verbose++;
	else if(VAGFilename[0] == '\0') strncpy(VAGFilename, argv[i], sizeof(VAGFilename) - 1);
			 else strncpy(WBHFilename, argv[i], sizeof(VAGFilename) - 1);
 }

 if(VAGFilename[0] == '\0') Usage();
 if(Verbose) printf("\n VAG-Com filename: %s\n", VAGFilename);
 
 if(WBHFilename[0] == '\0'){
	if(Verbose > 1) printf(" Creating WBH-Diag filename\n");
  CreateWBHFilename(VAGFilename, WBHFilename);
 }
 if(Verbose) printf(" WBH-Diag filename: %s\n", WBHFilename);

 Convert(VAGFilename, WBHFilename);

 return 0;
}
